package de.mattv.quarry

import net.minecraft.core.BlockPos
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TextComponent
import net.minecraft.world.InteractionHand
import net.minecraft.world.InteractionResultHolder
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.CreativeModeTab
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.TooltipFlag
import net.minecraft.world.level.ClipContext
import net.minecraft.world.level.Level

class QuarryController : Item(Properties().tab(CreativeModeTab.TAB_MISC).stacksTo(1)) {
    private var areaMode = false
    private var areaModeFirstPos: QuarryChunk? = null
    private var quarry: BlockPos? = null

    override fun use(level: Level, player: Player, hand: InteractionHand): InteractionResultHolder<ItemStack> {
        if (!level.isClientSide() && hand == InteractionHand.MAIN_HAND) {
            val item = player.getItemInHand(hand)
            load(item)
            if (player.isSecondaryUseActive) {
                val ray = getPlayerPOVHitResult(level, player, ClipContext.Fluid.NONE)
                val block = ray.blockPos
                val be = level.getBlockEntity(block)
                if (be is QuarryTile) {
                    quarry = block
                    player.displayClientMessage(TextComponent("Bound controller to ${block.x}, ${block.y}, ${block.z}"), true)
                    save(item)
                    return InteractionResultHolder.success(item)
                }
                areaMode = !areaMode
                if (!areaMode) areaModeFirstPos = null
                player.displayClientMessage(TextComponent(if (areaMode) "Mode: Area" else "Mode: Single"), true)
                save(item)
                return InteractionResultHolder.success(item)
            } else if (quarry != null) {
                val be = level.getBlockEntity(quarry!!)
                if (be is QuarryTile) {
                    val chunk = QuarryChunk(player.blockPosition())
                    if (areaMode && areaModeFirstPos != null) {
                        val chunk1 = areaModeFirstPos!!.min(chunk)
                        val chunk2 = areaModeFirstPos!!.max(chunk)
                        var chunkCount = 0
                        for (x in chunk1.x..chunk2.x step 16) for (z in chunk1.z..chunk2.z step 16) {
                            be.addChunk(QuarryChunk(x, z))
                            chunkCount++
                        }
                        player.displayClientMessage(TextComponent("Added $chunkCount chunks to quarry"), true)
                        save(item)
                        return InteractionResultHolder.pass(item)
                    } else if (areaMode) {
                        areaModeFirstPos = chunk
                        player.displayClientMessage(TextComponent("Set first position of area"), true)
                        save(item)
                        return InteractionResultHolder.pass(item)
                    } else {
                        be.addChunk(chunk)
                        player.displayClientMessage(TextComponent("Added chunk"), true)
                        save(item)
                        return InteractionResultHolder.pass(item)
                    }
                } else {
                    player.displayClientMessage(TextComponent("Bound quarry is invalid, please rebind"), true)
                    quarry = null
                    save(item)
                    return InteractionResultHolder.fail(item)
                }
            } else {
                player.displayClientMessage(TextComponent("Controller is not bound"), true)
                return InteractionResultHolder.fail(item)
            }
        }
        return super.use(level, player, hand)
    }

    override fun appendHoverText(stack: ItemStack, level: Level?, tooltip: MutableList<Component>, flagIn: TooltipFlag) {
        load(stack)
        tooltip.add(TextComponent(if (areaMode) "Mode: Area" else "Mode: Single"))
        if (areaModeFirstPos != null)
            tooltip.add(TextComponent("First chunk: ${areaModeFirstPos!!.x} ${areaModeFirstPos!!.z}"))
        if (quarry != null)
            tooltip.add(TextComponent("Quarry: ${quarry!!.x}, ${quarry!!.y}, ${quarry!!.z}"))
        super.appendHoverText(stack, level, tooltip, flagIn)
    }

    private fun save(stack: ItemStack) {
        val tag = CompoundTag()
        tag.putBoolean("area", areaMode)
        if (areaModeFirstPos != null)
            tag.put("areaFirst", areaModeFirstPos!!.save())
        if (quarry != null)
            tag.putIntArray("quarry", arrayListOf(quarry!!.x, quarry!!.y, quarry!!.z))

        stack.tag = tag
    }

    private fun load(stack: ItemStack) {
        val tag = stack.getOrCreateTag()
        if (tag.contains("area"))
            areaMode = tag.getBoolean("area")
        areaModeFirstPos =
            if (tag.contains("areaFirst")) QuarryChunk(tag.getCompound("areaFirst"))
            else null
        quarry =
            if (tag.contains("quarry")) {
                val arr = tag.getIntArray("quarry")
                BlockPos(arr[0], arr[1], arr[2])
            }
            else null
    }
}